﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Automation;
using System.Threading;

namespace Calculator
{
    /// <summary>
    /// This class is constructed to wait for a specific window to open and return the AutomationElement for that window
    /// </summary>
    public sealed class WindowOpenWaiter : IDisposable
    {
        private string windowTitle;
        private bool windowFound;
        private AutomationEventHandler UIAeventHandler;
        private AutomationElement element;
        AutoResetEvent[] autoEvents;

        /// <summary>
        /// The AutomationElement that was found
        /// </summary>
        public AutomationElement Element
        {
            get
            {
                return this.element;
            }
        }
        
        /// <summary>
        /// WindowOpenWaiter constructor
        /// </summary>
        /// <param name="windowTitle">The title of the window we are waiting for</param>
        public WindowOpenWaiter(string windowTitle)
        {
            if (String.IsNullOrEmpty(windowTitle))
            {
                throw new ArgumentNullException("windowTitle");
            }

            this.windowTitle = windowTitle;
        }

        /// <summary>
        /// Wait for the window to popup
        /// </summary>
        /// <param name="time">The amount of time to wait in ms</param>
        /// <exception cref="TimeoutException">Will return a timeout exception if no window where found in the time</exception>
        public void Wait(int time)
        {
            //Set aut event to false
            autoEvents = new AutoResetEvent[]
            {
                new AutoResetEvent(false)
            };

            UIAeventHandler = new AutomationEventHandler(OnWindowOpenedEvent);
            Automation.AddAutomationEventHandler(WindowPattern.WindowOpenedEvent, AutomationElement.RootElement, TreeScope.Descendants, UIAeventHandler);

            // Wait for auto event to be true
            WaitHandle.WaitAll(autoEvents, time, true);

            if (!windowFound)
            {
                throw new TimeoutException("Did timeout waiting for window with title '" + this.windowTitle + "'");
            }
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void OnWindowOpenedEvent(object src, AutomationEventArgs args)
        {
            try
            {
                this.element = src as AutomationElement;
            }
            catch
            {
                return;
            }

            string name = "";
            if (this.element == null)
            {
                name = "null";
            }
            else
            {
                name = this.element.GetCurrentPropertyValue(AutomationElement.NameProperty) as string;
            }

            if(name.Equals(this.windowTitle))
            {
                this.windowFound = true;
                autoEvents[0].Set();
            }
        }

        #region IDisposable Members

        public void Dispose()
        {
            try
            {
                Automation.RemoveAutomationEventHandler(WindowPattern.WindowOpenedEvent, AutomationElement.RootElement, UIAeventHandler);
            }
            catch (Exception)
            {                
                throw;
            }
            
        }

        #endregion
    }
}
